function [ imgDest ] = rotate( imgSrc, theta )
%function [ imgDest ] = rotate( imgSrc, theta )
%  Rotates a bitmap, imgIn, by theta radians to produce imgOut.
%  As usual, positive angles rotate the image clockwise.
%
%  ISE1 And EE2 Computing Course
%  MATLAB Assignment: Image Warping
%  Exercise 1 - Image Rotation
%
%  Andy Bennett (andyjpb@ashurst.eu.org), 16/2/2003

% wouldn't it be nice to be able to do something like:
%  y = 1:1:size(imgSrc,1);
%  x = 1:1:size(imgSrc,2);
%  imgDest(y,x) = (inv(transform) \ (imgSrc(y,x) - center)) + center;

% set up constants
invtransform = inv([cos(theta) sin(theta); -sin(theta) cos(theta)]);
center = round(size(imgSrc) / 2);

% use some more constants to simplify the expression for the actual transformation
A = invtransform(1,1);
B = invtransform(1,2);
C = invtransform(2,1);
D = invtransform(2,2);
Yc = center(1);
Xc = center(2);

% initialise the destination image to improve speed / memory efficiency
imgDest = zeros(size(imgSrc,1),size(imgSrc,2));

%imgDest(y,x) = imgSrc( round((C*(x - Xc) + D*(y - Yc)) + Yc) , round((A*(x - Xc) + B*(y - Yc)) + Xc) );

for y = 1:size(imgSrc,1),
  for x= 1:size(imgSrc,2),
    M = round((invtransform)*([x;y] - [Xc; Yc]) + [Xc; Yc])+1;
    if ((M(1) <= size(imgSrc,2)) & (M(1) > 0) & (M(2) <= size(imgSrc,1)) & (M(2) > 0))
      imgDest(y,x) = imgSrc(M(2),M(1));
    else
      imgDest(y,x) = 0;
    end
  end
end
